#ifndef _Pointer_H
#define _Pointer_H 1

#include <stdio.h>         // NULL

// Pointer Class definition
template<class T>
class Pointer {

 private:
  
    // structure to store general information 
    struct P_info {
      int  size;           // number of array elements (1 if scalar)
      int  counter;        // number of pointers to this same memory
    };
  
    P_info *p_info;        // pointer to a structure storing size and count
                           //  of allocated memory
  
    Pointer *left, *right; // double-link list linking all the pointers 
                           //  to the same memory
    int index;             // current index of array 
                           //  (0=<index<=size-1, 0 at start or scalar) 
    T *ptr;                // actual pointer to allocated memory

 public:
    
    // constructors and destructors
    Pointer();    // void (array) constructor
    Pointer(Pointer& p);   // copy constructor 
    Pointer(T* tptr);      // promoted copy constructor(for auto variable only)
    ~Pointer();            // destructor

    // memory allocator and deallocator
    Pointer& newptr(const int n = 1); // allocator
    Pointer& deleteptr(void);         // deallocator

    // pointer arithmatic operators
    T& operator *();                  // de-reference operator
    T* operator->();                  // member de-reference operator
    T& operator[](const int i);       // subscript de-reference operator

    Pointer operator+(const int n);   // addition    operator : ptr + n
    Pointer operator-(const int n);   // subtraction operator : ptr - n
    Pointer& operator++();            // prefix addition  operator : ++ptr
    Pointer operator++(int);          // postfix addition operator : ptr++
    Pointer& operator--();            // prefix subtraction  operator: --ptr
    Pointer operator--(int);          // postfix subtraction operator: ptr--

    // assignment operators
    Pointer& operator=(Pointer&);     // assignment operator (for the same type)
    Pointer& operator=(T*);           // assignment operator (for promoted type, 
                                      //  auto variable only)
}; 

#endif _Pointer_H
